﻿define("workspace/components/WorkspaceGrid", [
    "dojo",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "epi",
    "epi/dependency",
    "epi-cms/component/ContentQueryGrid",
    "epi-cms/dgrid/formatters",
    "workspace/commands/RemoveItem",
    "workspace/commands/SendProject",
    "workspace/commands/PublishItem",
    "workspace/commands/CancelProject",
    "workspace/commands/ReadClayTablet"
], function (dojo, declare, lang, epi, dependency, ContentQueryGrid, formatters, RemoveItem, SendProject, PublishItem, CancelProject, ReadClayTablet) {

    return declare([ContentQueryGrid], {
        constructor: function () {

            var self = this;

            //var refresh = function() {
            //    self.fetchData();
            //}

            //dojo.subscribe("workspace/events/updateprojects", refresh);
            //var timeOut = 5 * 1000;
            //setInterval(function () {
            //    dojo.publish("workspace/events/updateprojects");
            //}, timeOut);
        },
        sortKey: null,
        descending: false,
        postMixInProperties: function () {
            //RELEASE BUILD COMMENT console.log("entering postMixInProperties");
            this.inherited(arguments);

            //encapsulate the epi-server menu formatter to set the tooltip on the context menu
            var customFormatter = function (data, object) {
                //console.log(data);
                //console.log(object);
                //todo: find out where data for the formatter is set
                var menu = formatters.menu({ title: "Options"});
                return menu;
            };

            lang.mixin(this.defaultGridMixin, {
                columns: {
                    name: {
                        renderCell: function (object, value, node) {
                            //RELEASE BUILD COMMENT console.log("Rendering Name Column");
                            node.innerHTML = formatters.contentItem(object.typeIdentifier, object.missingLanguageBranch, value);
                        }
                    },
                    status: {
                        className: "epi-width16",
                        renderCell: function (object, value, node) {
                            //RELEASE BUILD COMMENT console.log("Rendering Status Column");
                            var registry = dependency.resolve("epi.storeregistry");
                            var store = registry.get("workspace.translationprojectstore");

                            var query = store.get(object.contentLink + "?" + Math.random() * 9999999999);

                            if (query.then) {
                                query.then(function (item) {
                                    //RELEASE BUILD COMMENT console.log("Return from translationprojectstore");

                                    var cssClass = item.status == "3" ? "epi-iconCheckmark" : "epi-iconClose";
                                    switch (item.status) {
                                        case 0:
                                            cssClass = "epi-iconShare";
                                            break;
                                        case 1:
                                        case 101:
                                        case 102:
                                            cssClass = "epi-iconClock";
                                            break;
                                        case 2:
                                            cssClass = "epi-iconDownload";
                                            break;
                                        case 3:
                                            cssClass = "epi-iconCheckmark epi-icon--success";
                                            break;
                                        default:
                                            break;
                                    }

                                    var title = "";

                                    if (item.error) {
                                        //todo: trigger alert on click on icon
                                        title = 'Something went wrong: ' + item.errorMessage;
                                        //var alertFunc = function() {
                                            

                                        //    ////var dialog = new Alert({ heading: 'Status error', description:  dialogMessage});
                                        //    ////dialog.show();
                                        //    //alert(dialogMessage);

                                        //}

                                        //node.onclick += alertFunc;
                                        cssClass = "epi-iconError";
                                    }


                                    
                                    var percentage = item.percentageCompleted == 0 ? '' : item.percentageCompleted + '%';
                                    node.innerHTML = "<span id=\"exampleNode\" class=\"dijitInline dijitIcon " + cssClass + "\" title=\"" + title + "\"></span><span>" + percentage + " </span>";
                                });
                            } else {
                                node.innerHTML = "nada";
                            }
                            

                            //this.store.remove(this.model.contentLink);
                            //this.store.query(object.contentLink);
                            //dojo.when(this.store.query(object.contentLink), function(returnValue) {
                            //    alert(returnValue);
                            //});
                            //$.get("/modules/app/Stores/translationprojectstore/" + object.contentLink, function (data) {
                            //    var cssClass = data == "{}&&3" ? "epi-iconCheckmark" : "epi-iconClose";
                            //    switch (parseInt(data.substr(data.length - 1))) {
                            //        case 0:
                            //            cssClass = "epi-iconCheckmark";
                            //            break;
                            //        case 1:
                            //            cssClass = "epi-iconClock";
                            //            break;
                            //        case 2:
                            //            cssClass = "epi-iconClose";
                            //            break;
                            //        case 3:
                            //            cssClass = "epi-iconStop";
                            //            break;
                            //        default:
                            //            break;
                            //    }
                            //    node.innerHTML = "<span class=\"dijitInline dijitIcon " + cssClass + "\"></span>";
                            //});
                        }
                    },
                    menu: {
                        className: "epi-width10",
                        formatter: customFormatter
                    }
                }
            });

            this.commands = [
                new SendProject({ store: this.store, contentQuery: this }),
                new ReadClayTablet({ store: this.store, contentQuery: this }),
                new PublishItem({ store: this.store, contentQuery: this }),
                new CancelProject({ store: this.store, contentQuery: this }),
                new RemoveItem({ store: this.store, contentQuery: this })
            ];
        },

        fetchData: function () {
            var queryOptions = { ignore: ["query"] };
            if (this.sortKey) {
                queryOptions.sort = [{ attribute: this.sortKey, descending: this.descending }];
            }
            this.grid.set("queryOptions", queryOptions);

            var queryParameters = this.queryParameters || {};
            queryParameters.query = this.queryName;
            //RELEASE BUILD COMMENT console.log(this.queryName);
            this.grid.set("query", queryParameters);

        },

        postCreate: function () {
            this.inherited(arguments);
            this.grid.contextMenu.addProvider(this);
        },

        _onSelect: function (e) {
            var item = e.rows[0].data;

            this.commands.forEach(function (command) {
                command.set("model", item);
            });
        }
    });
});
