﻿function NodeViewModel(data) {
    var self = this;

    self.find = function (id) {
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }

        return null;
    };
    
    self.id = ecore.ko.observable(data.id);
    self.isExpanded = ecore.ko.observable(false);
    self.selected = ecore.ko.observable(data.selected || ecore.viewModel.findAsset(data.id));
    self.name = ecore.ko.observable(data.name);
    self.nodes = ecore.ko.observableArray([]);
    self.selectAllTree = function() {
        self.selected(true);
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].selectAllTree();
    }
    self.deSelectAllTree = function () {
        self.selected(false);
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++)
            selfNodes[i].deSelectAllTree();
    }
    self.addAllTree = function () {
        ecore.viewModel.isLoading(true);
        var assets = ecore.treeStore.put(self.id() + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);
        if (assets.then) {
            assets.then(function (item) {
                ecore.viewModel.addAssetsFromTree(item.result);
                //ecore.viewModel.isLoading(false);
            });
        }
        
        self.selectAllTree();
    };
    self.removeAllTree = function () {
        ecore.viewModel.isLoading(true);
        var assets = ecore.treeStore.put(self.id() + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);
        if (assets.then) {
            assets.then(function (item) {
                ecore.viewModel.removeAssetsFromTree(item.result);
                //ecore.viewModel.isLoading(false);
            });
        }
        self.deSelectAllTree();
    };
    self.hasNodes = ecore.ko.observable(data.hasChildren);
    self.childrenLoaded = ecore.ko.observable(false);
    self.selected.subscribe(function (value) {
        if (value)
            ecore.viewModel.addAssetFromTree(self);
        else
            ecore.viewModel.removeAssetFromTree(self);
    });

    self.appendChildren = function (nodes) {
        self.childrenLoaded(true);

        for (var i = 0; i < nodes.length; i++)
            self.nodes.push(new NodeViewModel(nodes[i]));

        //var selfNodes = self.nodes();
    };

    if (data.nodes)
        self.appendChildren(data.nodes);

    self.toggleVisibility = function () {
        self.isExpanded(!self.isExpanded());

        if (self.isExpanded() && !self.childrenLoaded()) { // load new children
            self.childrenLoaded(true);
            ecore.loadChildNodes(self.id(), self);
        }
    };
};

function AssetViewModel() {
    var self = this;
    self.isLoading = ecore.ko.observable(false);
    self.isSaving = ecore.ko.observable(false);
    self.progress = ecore.ko.observable(0);
    self.progressTotal = ecore.ko.observable(0);
    self.assets = ecore.ko.observableArray([]);
    self.nodes = ecore.ko.observableArray([]);
    self.editable = ecore.ko.observable(false);
    self.addAssetFromTree = function (node, isNotObservable) {
        if (!isNotObservable) {
            self.assets.push({ id: node.id(), name: node.name() });
        } else {
            if (!ecore.viewModel.findAsset(node.id)) {
                self.assets.push({ id: node.id, name: node.name });
            }
        }
    };

    self.addAssetsFromTree = function (nodes) {
        var length = nodes.length;
        self.progressTotal(length);
        var index = 0;
        var process = function () {
            for (; index < length; index++) {
                var toProcess = nodes[index];
                self.addAssetFromTree(toProcess, true);
                if (index + 1 < length && index % 25 == 0) {
                    setTimeout(process, 100);
                    self.progress(index);
                    index++;
                    break;
                }
                if (index + 1 == length) {
                    ecore.viewModel.isLoading(false);
                }
            }
        }
        process();
    };
    self.removeAssetsFromTree = function (nodes) {
        var length = nodes.length;
        self.progressTotal(length);
        var index = 0;
        var process = function () {
            for (; index < length; index++) {
                var toProcess = nodes[index];
                self.removeAssetFromTree(toProcess, true);
                if (index + 1 < length && index % 25 == 0) {
                    setTimeout(process, 100);
                    self.progress(index);
                    index++;
                    break;
                }
                if (index + 1 == length) {
                    ecore.viewModel.isLoading(false);
                }
            }
        }
        process();
    };
    self.find = function (id) {
        var selfNodes = self.nodes();
        for (var i = 0; i < selfNodes.length; i++) {
            if (selfNodes[i].id() == id)
                return selfNodes[i];
            else {
                var result = selfNodes[i].find(id);
                if (result != null)
                    return result;
            }
        }

        return null;
    };
    self.findAsset = function(id) {
        var selfAssets = self.assets();
        for (var i = 0; i < selfAssets.length; i++) {
            if (selfAssets[i].id == id)
                return selfAssets[i];
        }
        return null;
    };
    
    self.removeAssetFromTree = function (node, isNotObservable) {
        var assets = self.assets();

        for (var i = 0; i < assets.length; i++)
            if (assets[i].id == (isNotObservable ? node.id : node.id()))
                self.removeAsset(assets[i]);
    };
    self.deselectInTree = function (asset) {
        var node = self.find(asset.id);

        if (node)
            node.selected(false);
        //else
        self.removeAsset(asset);
    };
    self.removeAsset = function (asset) {
        self.assets.remove(asset);
    };
    self.save = function () {
        ecore.saveAssets();
    };

    self.close = function () {
        ecore.closeDialog();
    };

    self.appendChildren = function (nodes) {
        for (var i = 0; i < nodes.length; i++)
            self.nodes.push(new NodeViewModel(nodes[i]));

        var selfNodes = self.nodes();

        // load whole tree
        for (var i = 0; i < selfNodes.length; i++)
            if (!selfNodes[i].childrenLoaded()) {
                selfNodes[i].childrenLoaded(true);
                ecore.loadChildNodes(selfNodes[i].id(), selfNodes[i]);
            }
    };
}

window["ecore"] = {
    epieditor: null,
    dataStore: null,
    treeStore: null,
    viewModel: null,
    ko: null,
    _projectId: -1,
    dialog: null,
    inited: false,

    init: function (epieditor, dataStore, treeStore, ko) {
        var self = this;
        self.dataStore = dataStore;
        self.epieditor = epieditor;
        self.treeStore = treeStore;
        self.ko = ko;

        if (!self.dialog)
            self.dialog = new dijit.Dialog({ content: dojo.cache("workspace.editors", "MultiSelectEditorLightbox.html"), style: 'width: 90%; height: 95%', title: 'Files in project' });

        if (!self.inited) {
            self.createNewModel();
            self.bindModel();
            self.inited = true;
        }
    },

    getProjectId: function () {
        if (this._projectId == -1)
            this._projectId = $('#lb-project-id').val();

        return this._projectId;
    },

    createNewModel: function () {
        this.viewModel = new AssetViewModel();
    },

    showDialog: function () {
        var self = this;
        self.emptyModel();
        self.loadAssets();
        self.loadRootNodes();
        self.dialog.show();
        self.loadDialog(this.dialog);
        console.log('Loading project: ' + self.getProjectId());
    },

    loadRootNodes: function () {
        this.viewModel.nodes([]);
        this.loadChildNodes(-1, this.viewModel);
    },

    loadChildNodes: function (parentID, model) {
        var self = this;
        var query = self.treeStore.get(parentID + ";" + ecore.getProjectId() + "?" + Math.random() * 9999999999);

        if (query.then) {
            query.then(function (item) {
                model.appendChildren(item.result);
            });
        } else {
            model.appendChildren(query.result);
        }
    },

    saveAssets: function () {
        var self = this;
        var data = self.getProjectId();
        var assets = self.viewModel.assets();

        for (var i = 0; i < assets.length; i++)
            data += '_' + assets[i].id;

        ecore.viewModel.isSaving(true);
        self.dataStore.put(data).then(function (response) {
            ecore.closeDialog();
            ecore.viewModel.isSaving(false);
        });
    },

    closeDialog: function () {
        this.dialog.hide();
    },

    emptyModel: function () {
        this._projectId = -1;
        this.viewModel.assets([]);
        this.viewModel.nodes([]);
    },

    bindModel: function () {
        this.ko.applyBindings(this.viewModel, document.getElementById("lb-editor"));
    },

    unbindModel: function () {
        var self = this;
        var node = jQuery("#lb-editor");
        node.find("*").each(function () {
            jQuery(this).unbind();
        });

        ko.cleanNode(document.getElementById("lb-editor"));
    },

    loadDialog: function (dialog) {
        var self = this;
        var total = jQuery('.dijitDialog').outerHeight();
        var title = jQuery('.dijitDialogTitleBar').outerHeight();
        var size = total - title;
        jQuery('.dijitDialogPaneContent').css('height', size + 'px');
        jQuery('.dijitDialogPaneContent').css('padding', '0px');
        jQuery('.lb-pane').css('max-height', (size - 112) + 'px');
        jQuery('.lb-pane').css('height', (size - 112) + 'px');
        self.dialog = dialog;
    },

    loadAssets: function () {
        var self = this;
        var query = self.dataStore.get(self.getProjectId() + "?" + Math.random() * 9999999999);

        if (query.then) {
            query.then(function (item) {
                self.viewModel.editable(item.result.editable);
                self.viewModel.assets(item.result.assets);
            });
        }
        else {
            self.viewModel.editable(query.result.editable);
            self.viewModel.assets(query.result.assets);
        }
    }
};